@@ -40,8 +40,8 @@ def create_ssh_cmd(sshuser, host):
     """SSH function that returns SSH string"""
 
     connstring = "%s -i %s -o UserKnownHostsFile=%s %s@%s " % (
-                        config.get('System', 'ssh'), 
-                        PEM_FILE, 
+                        config.get('System', 'ssh'),
+                        PEM_FILE,
                         HOST_FILE,
                         sshuser, host
                     )
@@ -108,12 +108,29 @@ def install_datastax_ssh(user):
             scp_send(user, ip, datastax_ssh)
             exe_ssh_cmd(create_ssh_cmd(user, ip), 'chmod +x datastax_ssh; sudo mv datastax_ssh /usr/bin/datastax_ssh; sudo mkdir /etc/cassandralauncher; sudo mv nodelist /etc/cassandralauncher/nodelist')
 
+def upload_smoke_tests(user):
+    # Find the datastax_ssh original file
+    qa_file = os.path.join(os.path.dirname(__file__), '../', 'qa', 'run_packaging_smoke.sh')
+    if not os.path.exists(qa_file):
+        return
+
+    # Send files to the cluster
+    for ip in public_ips:
+        scp_send(user, ip, qa_file, 'run_packaging_smoke.sh')
+        exe_ssh_cmd(create_ssh_cmd(user, ip), 'chmod +x run_packaging_smoke.sh')
+        # exe_ssh_cmd(create_ssh_cmd(user, ip), 'sudo ./run_packaging_smoke.sh hadoop pkg')
+        # exe_ssh_cmd(create_ssh_cmd(user, ip), 'sudo ./run_packaging_smoke.sh search pkg')
+
 def install_hosts_appending(user):
     # Write the public IPs to a hosts file
     with tempfile.NamedTemporaryFile() as tmp_file:
         hosts_file = ''
         for i, ip in enumerate(private_ips):
-            hosts_file += '{0:15} {1}\n'.format(ip, 'node{0}'.format(i))
+            for node_type in node_types:
+                if i in node_types[node_type]:
+                    i = node_types[node_type].index(i)
+                    break
+            hosts_file += '{0:15} {1}\n'.format(ip, '{0}{1}'.format(node_type, i))
         tmp_file.write(hosts_file)
         tmp_file.flush()
 
@@ -191,9 +208,10 @@ def prime_connections(public_ips, user):
 
     print "Priming connections..."
 
-    # Clear previous file
-    with open(HOST_FILE, 'w') as f:
-        f.write('')
+    # Clear previous file, if not running parallel tests
+    if not cli_options['CLI_qa']:
+        with open(HOST_FILE, 'w') as f:
+            f.write('')
 
     accept_rsa_fingerprints = config.get('Cassandra', 'accept_rsa_fingerprints')
 
@@ -217,7 +235,7 @@ def prime_connections(public_ips, user):
         # If performing individual authentication, prompt
         if accept_rsa_fingerprints != 'all':
             accept_rsa_fingerprints = prompt_rsa_fingerprints(ip, fingerprint)
-        
+
         # Append all public RSA keys into HOST_FILE
         with open(HOST_FILE, 'a') as f:
             f.write(rsa_key)
@@ -347,6 +365,11 @@ options_tree = {
         'Prompt': 'Install Opscenter',
         'Help': 'True | False'
     },
+    'release': {
+        'Section': 'Cassandra',
+        'Prompt': 'Specify Package Release',
+        'Help': 'Package Release'
+    },
     'username': {
         'Section': 'Cassandra',
         'Prompt': 'DataStax Username',
@@ -357,10 +380,15 @@ options_tree = {
         'Prompt': 'Uses getpass()',
         'Help': 'DataStax Password'
     },
-    'realtimenodes': {
+    'analyticsnodes': {
         'Section': 'Cassandra',
-        'Prompt': 'Realtime (Non-Analytic) Nodes',
-        'Help': 'Number of Vanilla Cassandra Nodes'
+        'Prompt': 'Analytics Nodes',
+        'Help': 'Number of Analytics Nodes'
+    },
+    'searchnodes': {
+        'Section': 'Cassandra',
+        'Prompt': 'Search Nodes',
+        'Help': 'Number of Search Nodes'
     },
     'cfsreplicationfactor': {
         'Section': 'Cassandra',
@@ -387,6 +415,12 @@ options_tree = {
         'Prompt': 'NoPrompts',
         'Action': 'store_true',
         'Help': 'Ensures that no user prompts will occur.'
+    },
+    'qa':{
+        'Section': 'CLI',
+        'Prompt': 'QA',
+        'Action': 'store_true',
+        'Help': 'Upload QA scripts.'
     }
 }
 
@@ -485,17 +519,15 @@ def main():
         sys.stderr.write("None of the configurations should be wrapped in quotes.\n")
         sys.stderr.write("    EC2:aws_access_key_id or EC2:aws_secret_access_key appears to be.\n")
         sys.exit(1)
-    ec2.terminate_cluster(check_cascading_options('aws_access_key_id'), check_cascading_options('aws_secret_access_key'), config.get('EC2', 'placement'), check_cascading_options('handle'), prompt_continuation=True)
+
+    if not cli_options['CLI_noprompts']:
+        ec2.terminate_cluster(check_cascading_options('aws_access_key_id'), check_cascading_options('aws_secret_access_key'), config.get('EC2', 'placement'), check_cascading_options('handle'), prompt_continuation=True)
 
     start_time = time.time()
 
     # Get basic information for both Community and Enterprise clusters
     clustername = check_cascading_options('clustername')
-    if True:
-        clustername = clustername.replace("'", "").replace(' ', '')
-    else:
-        # AMI 2.2 allows spaces
-        clustername = "'%s'" % clustername.replace("'", "")
+    clustername = "'%s'" % clustername.replace("'", "")
 
     # Ensure totalnodes > 0
     ignore_command_line = False
@@ -506,11 +538,11 @@ def main():
         else:
             config.set('Cassandra', 'totalnodes')
             ignore_command_line = True
-    
-    version = check_cascading_options('version', choices=['Community', 'Enterprise'])
+
+    version = check_cascading_options('version', choices=['Community', 'Enterprise']).title()
     user_data = '--clustername %s --totalnodes %s --version %s' % (clustername, totalnodes, version)
 
-    if version.lower() == 'Enterprise'.lower():
+    if version == 'Enterprise':
         ignore_command_line = False
         while True:
             # Get additional information for Enterprise clusters
@@ -526,32 +558,45 @@ def main():
                 ignore_command_line = True
                 print "Authentication to DataStax server failed. Please try again."
 
-        # Check the number of Vanilla Cassandra nodes that will launch
+        # Check the number of Analytics Nodes that will launch
         ignore_command_line = False
         while True:
-            realtimenodes = check_cascading_options('realtimenodes', int, ignore_command_line=ignore_command_line)
-            if realtimenodes <= totalnodes:
+            analyticsnodes = check_cascading_options('analyticsnodes', int, ignore_command_line=ignore_command_line)
+            if analyticsnodes <= totalnodes:
                 break
             else:
+                print "Overallocation of the chosen %d nodes" % (totalnodes)
                 # Clear the previous cfsreplicationfactor
-                config.set('Cassandra', 'realtimenodes')
+                config.set('Cassandra', 'analyticsnodes')
                 ignore_command_line = True
-        
-        user_data += ' --username %s --password %s --realtimenodes %s' % (username, password, realtimenodes)
+
+        # Check the number of Search Nodes that will launch
+        ignore_command_line = False
+        while True:
+            searchnodes = check_cascading_options('searchnodes', int, ignore_command_line=ignore_command_line)
+            if analyticsnodes + searchnodes <= totalnodes:
+                break
+            else:
+                print "Overallocation of the chosen %d nodes" % (totalnodes)
+                # Clear the previous cfsreplicationfactor
+                config.set('Cassandra', 'searchnodes')
+                ignore_command_line = True
+
+        user_data += ' --username %s --password %s --analyticsnodes %s --searchnodes %s' % (username, password, analyticsnodes, searchnodes)
 
         # If Hadoop enabled nodes are launching, check the CFS replication factor
-        analyticnodes = totalnodes - realtimenodes
-        if analyticnodes > 0:
+        if analyticsnodes > 0:
             ignore_command_line = False
             while True:
                 cfsreplicationfactor = check_cascading_options('cfsreplicationfactor', int, ignore_command_line=ignore_command_line)
-                if cfsreplicationfactor >= 1 and cfsreplicationfactor <= analyticnodes:
+                if 1 <= cfsreplicationfactor and cfsreplicationfactor <= analyticsnodes:
                     break
                 else:
+                    print "1 <= CFS Replication Factor <= Number of Analytics Nodes"
                     # Clear the previous cfsreplicationfactor
                     config.set('Cassandra', 'cfsreplicationfactor')
                     ignore_command_line = True
-        
+
             user_data += ' --cfsreplicationfactor %s' % (cfsreplicationfactor)
         print
 
@@ -566,6 +611,9 @@ def main():
     if check_cascading_options('installopscenter', optional=True) == 'False':
         user_data += ' --opscenter no'
 
+    if check_cascading_options('release', optional=True):
+        user_data += ' --release %s' % check_cascading_options('release')
+
     # DataStax AMI specific options and formatting
     image = check_cascading_options('datastax_ami', optional=True)
     if not image:
@@ -598,14 +646,40 @@ def main():
 
     start_priming(user)
 
+    if version == 'Enterprise':
+        global seed_index
+        realtimenodes = totalnodes - analyticsnodes - searchnodes
+        seed_index = [0, realtimenodes, realtimenodes + analyticsnodes]
+        seed_index.reverse()
+
+    global node_types
+    node_type = 'c'
+    node_types = {'c': [], 'a':[], 's':[]}
+
     print 'Primed Connection Strings:'
-    for node in public_ips:
-        print '{0} -i {1} -o UserKnownHostsFile={2} {3}@{4}'.format(config.get('System', 'ssh'), PEM_FILE, HOST_FILE, user, node)
+    for i, node in enumerate(public_ips):
+        if version == 'Enterprise' and i in seed_index:
+            if seed_index.index(i) == 0:
+                print 'Search (Solr) Nodes:'
+                node_type = 's'
+            if seed_index.index(i) == 1:
+                print 'Analytics (Hadoop) Nodes:'
+                node_type = 'a'
+            if seed_index.index(i) == 2:
+                print 'Realtime (Cassandra) Nodes:'
+                node_type = 'c'
+
+        node_types[node_type].append(i)
+        print '    {0} -i {1} -o UserKnownHostsFile={2} {3}@{4}'.format(config.get('System', 'ssh'), PEM_FILE, HOST_FILE, user, node)
     print
 
     print 'Installing DataStax SSH on the cluster...'
     install_datastax_ssh(user)
 
+    if cli_options['CLI_qa']:
+        print 'Uploading smoke tests on the cluster...'
+        upload_smoke_tests(user)
+
     print 'Setting up the hosts file for the cluster...'
     install_hosts_appending(user)
 
@@ -615,7 +689,12 @@ def main():
     print 'Total Elapsed Time: %s minutes %s seconds' % (end_time / 60, end_time % 60)
     print
 
-    ec2.terminate_cluster(check_cascading_options('aws_access_key_id'), check_cascading_options('aws_secret_access_key'), config.get('EC2', 'placement'), check_cascading_options('handle'))
+    if not cli_options['CLI_noprompts']:
+        ec2.terminate_cluster(check_cascading_options('aws_access_key_id'), check_cascading_options('aws_secret_access_key'), config.get('EC2', 'placement'), check_cascading_options('handle'))
+    else:
+        # Ensure the agents have time to start
+        # installing before exiting the program
+        time.sleep(10)
 
 def run():
     try:
